#! /bin/bash
#
# MySQL dump using MySQL root user - call as Linux root user!
# Originally from Alex Hornyak <alex0802h@gmail.com> (THX!).
# Part of mysql-addons package.
#
# Usage: mysql_dump.sh [backup_dir [backup_owner [backup_group]]]

compress="-c"                           # Compressed backup needed.
datetime=`date +%Y%m%d%H%M%S`           # Timestamp used in filenames.

# Need to be the Linux root.
if [ ! $UID -eq 0 ]; then echo "Run as root, please." >&2; exit 1; fi

# Getting parameters.
# Optional backup directory pathname defaults to ./timestamp.
dumppath=${1:-$(pwd)/$datetime}; shift
# Optional Linux user (backups owner) defaults to 'backup'.
dumpuser=${1:-"backup"}; shift
# Optional Linux owners group defaults to 'disk'.
dumpgroup=${1:-"disk"}; shift

# Checking components.
# MySQL command line call as MySQL root user.
mysql=$(which mysql); if [ -z "$mysql" -o ! -x "$mysql" ]; then echo "Missing MySQL binary." >&2; exit 1; fi
if [ ! -e "/etc/mysql/root.cnf" ]; then echo "Missing root.cnf file." >&2; exit 1; fi
mysql_root="env LANG=C $mysql --defaults-file=/etc/mysql/root.cnf"
# Database dump script call.
mysqldump=$(which mysqldump); if [ -z "$mysqldump" -o ! -x "$mysqldump" ]; then echo "Missing mysqldump binary." >&2; exit 1; fi
mysqldumpdb="/usr/local/sbin/mysql_dumpdb"; if [ ! -x "$mysqldumpdb" ]; then echo "Missing $mysqldumpdb script." >&2; exit 1; fi
mysqldump_root="env LANG=C $mysqldumpdb"

echo "$datetime $0 started."
# List of existing databases.
dblist=`$mysql_root -N --batch --execute='show databases;' `
if [ -n "$dblist" ]; then
    # Creating backup's directory is necessary.
    if [ ! -d "$dumppath" ]; then mkdir -m 2750 "$dumppath"; chown $dumpuser:$dumpgroup "$dumppath"; fi
    # Doing some cleanup.
    rm "$dumppath"/*.sql "$dumppath"/*.sql.gz 2>/dev/null
    # Enumerating databases and doing dump via script call.
    for dbname in $dblist
    do
        filename="$dumppath/$(hostname).$datetime.$dbname.sql"
        $mysqldump_root $compress $dbname "$filename" $dumpuser $dumpgroup
    done
fi
echo `date +%Y%m%d%H%M%S`" $0 finished."
