#! /bin/bash
#
# Repairing broken MyISAM, ARCHIVE, CSV MySQL tables, only checking InnoDB tables
# (that isn't possible to repair it automatically) using MySQL root user's account.
# Call as Linux root user! Part of mysql-addons package.
#
# Usage: mysql_repair.sh

CUT=$(which cut)			# cut command call
GREP=$(which grep)                      # grep command call
EXIT_ERR=1                              # Return code on error
SED=$(which sed)			# sed command call

# Need to be root.
if [ ! $UID -eq 0 ]; then echo "Run as root, please." >&2; exit $EXIT_ERR; fi

# MySQL command line call as MySQL root user.
if [ ! -e "/etc/mysql/root.cnf" ]; then echo "Missing root.cnf file." >&2; exit $EXIT_ERR; fi
mysql_root="env LANG=C /usr/bin/mysql --defaults-file=/etc/mysql/root.cnf"

# List of existing databases.
dblist=`$mysql_root -N --batch --execute="show databases;" 2>/dev/null`
if [ -n "$dblist" ]; then
    # Enumerating databases.
    for dbname in $dblist
    do
        # Enumerating tables.
        tblist=`$mysql_root -N --batch --execute="use $dbname; show tables;" 2>/dev/null`
        for tablename in $tblist
        do
	    engine=`$mysql_root -N --batch --execute="use $dbname; show table status where Name = \"$tablename\"" 2>/dev/null | $CUT -f2`
	    if [[ "$engine" =~ (MyISAM|ARCHIVE|CSV) ]]; then
                # Repair table for supported tables.
                result=`$mysql_root -N --batch --execute="repair table $dbname.$tablename;" 2>/dev/null`
                if [ ! -z "$result" ]; then  echo -e "$result"
		else echo -e "$dbname.$tablename\trepair\twarning\tEXCEPTION"; fi
	    elif [[ "$engine" =~ (InnoDB) ]]; then
		# Check table for InnoDB.
                result=`$mysql_root -N --batch --execute="check table $dbname.$tablename;" 2>/dev/null`
                if [ ! -z "$result" ]; then  echo -e "$result"
		else echo -e "$dbname.$tablename\tcheck\twarning\tEXCEPTION"; fi
	    else
		# A simple note for others.
		echo -e "$dbname.$tablename\tskip\tnote\t$engine UNSUPPORTED"
	    fi
        done
    done
fi
